package ru.amse.cat.evlarchick.ui;

import ru.amse.cat.evlarchick.textstructure.ISegment;
import ru.amse.cat.evlarchick.textstructure.ITextInfo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.io.Reader;
import java.io.IOException;

public class HotTextPane extends JScrollPane {
    private static int MY_X_INDENT = 10;
    private static int MY_Y_INDENT = 10;

    private TextPanel myTextPanel;
    private List<ISegmentListener> myListeners;

    public HotTextPane(ITextInfo textInfo) {
        super(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        myListeners = new ArrayList<ISegmentListener>();
        myTextPanel = new TextPanel(textInfo);
        setViewportView(myTextPanel);
        addComponentListener(new SizeChangeListener());
    }

    public void addSegmentListener(ISegmentListener segmentListener) {
        myListeners.add(segmentListener);
    }

    public void setLinesWereModified() {
        myTextPanel.setLinesWereModified();
    }

    public void setMarked(ISegment s) {
        myTextPanel.setMarked(s);
    }

    private class SizeChangeListener implements ComponentListener {

        public void componentResized(ComponentEvent e) {
            myTextPanel.setLinesWereModified();
            repaint();
        }

        public void componentMoved(ComponentEvent e) {
        }

        public void componentShown(ComponentEvent e) {
        }

        public void componentHidden(ComponentEvent e) {
        }
    }

    private class TextPanel extends JPanel {
        private Color myRimColor = Color.DARK_GRAY;
        private Color myBorderColor = Color.PINK;
        private Color myHighlightColor = Color.LIGHT_GRAY;
        private Color myForegroundColor = Color.BLACK;
        private Color myBackgroundColor = Color.WHITE;
        private Color mySegmentColor = new Color(240,255,240);
        private Color myDeadZoneColor = new Color(255, 220, 220);
        private Color mySegBorderColor = new Color(0, 94, 0);
        private Color myDZBorderColor = new Color(94, 0, 0);

        private int myLineLength;
        private int myCharWidth;
        private int myLineHeight;
        private boolean myWasModified;


        private Font myFont;
        private ISegment myMarked;
        private ITextInfo myTextInfo;
        private List<ILineInfo> myLineInfo;

        private int[] myHighlightBegin = new int[] {-1, -1};
        private int[] myHighlightEnd = new int[] {-1, -1};
        private int[] myHighlightMinPossible = new int[2];
        private int[] myHighlightMaxPossible = new int[2];

        public TextPanel(ITextInfo info) {
            myTextInfo = info;
            myLineInfo = new ArrayList<ILineInfo>();
            myFont = new Font("Monospaced", Font.PLAIN, 12);
            myWasModified = true;
            TextListener tl = new TextListener();
            addMouseListener(tl);
            addMouseMotionListener(tl);
        }

        public void setLinesWereModified() {
            myWasModified = true;
        }

        public void setMarked(ISegment s) {
            myMarked = s;
        }

        public void invalidateLinesInfo() {
            if (!myWasModified) {
                return;
            }
            myWasModified = false;
            myLineInfo.clear();
            Reader r = myTextInfo.getReader();
            try {
                r.reset();
            } catch (IOException e) {
                return;
            }
            char[] buffer = new char[myLineLength];
            int lastLineStart = 0;
            int counter = 0;
            int count = 0;
            int lastSpace = -1;
            int read;
            char c;

            try {
                while ((read = r.read()) != -1) {
                    c = (char) read;
                    buffer[count] = c;
                    counter++;
                    count++;

                    if (c == '\n') {
                        myLineInfo.add(new LineInfo(lastLineStart, count - 1,
                                new String(buffer, 0, count - 1)));
                        lastSpace = -1;
                        count = 0;
                        lastLineStart = counter;
                        continue;
                    } else if (Character.isWhitespace(c)) {
                        lastSpace = count;
                    }

                    if (count == myLineLength) {
                        c = (char)r.read();
                        counter++;

                        if (c == -1) {
                            myLineInfo.add(new LineInfo(lastLineStart, myLineLength,
                                    new String(buffer, 0, myLineLength)));
                            break;
                        } else if (c == '\n') {
                            myLineInfo.add(new LineInfo(lastLineStart, myLineLength,
                                    new String(buffer, 0, myLineLength)));
                            count = 0;
                            lastSpace = -1;
                            lastLineStart = counter;
                        } else if (Character.isWhitespace(c)) {
                            myLineInfo.add(new LineInfo(lastLineStart, myLineLength,
                                    new String(buffer, 0, myLineLength)));
                            count = 0;
                            lastSpace = -1;
                            lastLineStart = counter;
                        } else {
                            if (lastSpace == -1) {
                                myLineInfo.add(new LineInfo(lastLineStart, lastSpace,
                                        new String(buffer, 0, count)));
                                count = 1;
                                buffer[0] = c;

                            } else {
                                myLineInfo.add(new LineInfo(lastLineStart, lastSpace,
                                        new String(buffer, 0, lastSpace)));
                                move(buffer, lastSpace);
                                count = myLineLength - lastSpace + 1;
                                buffer[count - 1] = c;
                            }
                            lastLineStart = counter - count;
                            lastSpace = -1;
                        }
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            Dimension dimension = new Dimension((int)HotTextPane.this.getSize().getWidth(),
                    2 * MY_Y_INDENT + myLineInfo.size() * myLineHeight);
            setSize(dimension);
            setPreferredSize(dimension);
        }

        private void move(char[] buffer, int beginIndex) {
            for (int i = beginIndex; i < buffer.length; i++) {
                buffer[i - beginIndex] = buffer[i];
            }
            for (int i = buffer.length - beginIndex; i < buffer.length; i++) {
                buffer[i] = ' ';
            }
        }

        public void setSize(int width, int height) {
            super.setSize(width, height);
        }

        public void setSize(Dimension d) {
            super.setSize(d);
        }

        public void paintComponent(Graphics g) {
            Dimension size = getSize();
            int height = (int)size.getHeight();
            int width = (int)HotTextPane.this.getSize().getWidth();
            g.setFont(myFont);
            FontMetrics fm = g.getFontMetrics(myFont);
            myCharWidth = (int)fm.getStringBounds("a", g).getWidth();
            myLineHeight = fm.getAscent() + fm.getDescent() + fm.getLeading();
            myLineLength = (width - 2 * MY_X_INDENT) / myCharWidth;
            invalidateLinesInfo();

            g.setColor(myRimColor);
            g.fillRect(0, 0, width + 1, height + 1);
            g.setColor(myBackgroundColor);
            g.fillRect(MY_X_INDENT - 5, MY_Y_INDENT - 5,
                    width - 2 * MY_X_INDENT + 10, height - 2 * MY_Y_INDENT + 10);
            g.setColor(myBorderColor);
            g.drawRect(MY_X_INDENT - 8, MY_Y_INDENT - 8,
                    width - 2 * MY_X_INDENT + 16, height - 2 * MY_Y_INDENT + 16);

            paintHighlight(g);
            paintSegments(g);
            paintText(g);
        }

        private void paintText(Graphics g) {
            g.setColor(myForegroundColor);
            int height = MY_Y_INDENT + myFont.getSize();

            for (ILineInfo lineInfo : myLineInfo) {
                g.drawString(lineInfo.getText(), MY_X_INDENT, height);
                height += myLineHeight;
            }
        }

        private void paintHighlight(Graphics g) {

            if (myHighlightBegin[0] == -1)  {
                return;
            }
            g.setColor(myHighlightColor);
            int[] begin = myHighlightBegin;
            int[] end = myHighlightEnd;

            int[] temp;
            if ((begin[1] > end[1]) ||
                    (begin[1] == end[1] &&
                            begin[0] > end[0])){
                temp = end;
                end = begin;
                begin = temp;
            }

            if (begin[1] == end[1]) {
                g.fillRect(MY_X_INDENT + begin[0] * myCharWidth, MY_Y_INDENT + myLineHeight * begin[1],
                        myCharWidth * (end[0] - begin[0] + 1), myLineHeight);
            } else {
                ILineInfo cur = myLineInfo.get(begin[1]);
                g.fillRect(MY_X_INDENT + begin[0] * myCharWidth, MY_Y_INDENT + myLineHeight * begin[1],
                        myCharWidth * (cur.getLength() - begin[0]), myLineHeight);
                for (int i = begin[1] + 1; i < end[1]; i++) {
                    cur = myLineInfo.get(i);
                    g.fillRect(MY_X_INDENT, MY_Y_INDENT + i * myLineHeight,
                            myCharWidth * cur.getLength(), myLineHeight);
                }
                g.fillRect(MY_X_INDENT, MY_Y_INDENT + end[1] * myLineHeight,
                        myCharWidth * (end[0] + 1), myLineHeight);
            }
        }

        private void paintSegments(Graphics g) {
            paintSegmentsList(g, myTextInfo.getDeadZones(), false);
            paintSegmentsList(g, myTextInfo.getSegments(), true);
        }

        private void paintSegmentsList(Graphics g, List<ISegment> segments, boolean withTranslation) {
            Color borderColor = (withTranslation) ? mySegBorderColor
                    : myDZBorderColor;
            Color backgroundColor = (withTranslation) ? mySegmentColor
                    : myDeadZoneColor;

            Iterator<ILineInfo> iterator = myLineInfo.iterator();
            if (!iterator.hasNext()) {
                return;
            }
            ILineInfo curInfo = iterator.next();

            int height = MY_Y_INDENT;
            for (ISegment s : segments) {
                if (s.equals(myMarked)) {
                    backgroundColor = Color.YELLOW;
                }

                int start = s.getStart();
                int end = s.getEnd();


                while (start >= curInfo.getStartIndex() + curInfo.getLength()) {
                    curInfo = iterator.next();
                    height += myLineHeight;
                }
                start -= curInfo.getStartIndex();

                if (end <= curInfo.getStartIndex() + curInfo.getLength() + 1) {
                    end -= curInfo.getStartIndex();
                    paintFragment(g, backgroundColor, start, height, end, withTranslation, borderColor);
                } else {
                    paintFragment(g, backgroundColor, start, height, curInfo.getLength(), withTranslation, borderColor);
                    curInfo = iterator.next();
                    height += myLineHeight;

                    while (end > curInfo.getStartIndex() + curInfo.getLength() + 1) {
                        paintFragment(g, backgroundColor, 0, height, curInfo.getLength(), withTranslation, borderColor);
                        curInfo = iterator.next();
                        height += myLineHeight;
                    }

                    paintFragment(g, backgroundColor, 0, height, end - curInfo.getStartIndex(), withTranslation, borderColor);
                }
                backgroundColor = (withTranslation) ? mySegmentColor
                        : myDeadZoneColor;
            }
        }

        private void paintFragment(Graphics g, Color backgroundColor, int start, int height, int end, boolean withTranslation,
                                   Color borderColor) {
            g.setColor(backgroundColor);
            g.fillRect(MY_X_INDENT + start * myCharWidth, height,
                    (end - start) * myCharWidth , myLineHeight);
            if (withTranslation) {
                g.setColor(borderColor);
                g.drawRect(MY_X_INDENT + start * myCharWidth, height,
                        (end - start) * myCharWidth , myLineHeight);
            }
        }

        private int getIndexByPoint(Point p) {
            int x = (int)p.getX();
            int y = (int)p.getY();

            if (x <= MY_X_INDENT || y <= MY_Y_INDENT) {
                return -1;
            }
            int column = (x - MY_X_INDENT) / myCharWidth;
            int row = (y - MY_Y_INDENT) / myLineHeight;

            if (row >= myLineInfo.size()) {
                return -1;
            }

            ILineInfo lineInfo = myLineInfo.get(row);
            if (column < lineInfo.getLength()) {
                return lineInfo.getStartIndex() + column;
            }
            return -1;
        }

        private void dropHighlights() {
            myHighlightBegin[0] = -1;
            myHighlightBegin[1] = -1;
            myHighlightEnd[0] = -1;
            myHighlightEnd[0] = -1;
        }

        private class TextListener extends MouseAdapter implements MouseMotionListener {
            public void mouseClicked(MouseEvent me) {
                int index = getIndexByPoint(me.getPoint());
                if (index == -1) {
                    return;
                }

                int sIndex = myTextInfo.getSegmentIndexByTextPosition(index);
                if (sIndex != -1) {
                    for (ISegmentListener listener : myListeners) {
                        listener.segmentClicked(sIndex, false);
                        repaint();
                        if (me.getClickCount() > 1) {
                            listener.segmentTranslationChanged(sIndex);
                            repaint();
                        }
                    }
                }

                sIndex = myTextInfo.getDeadZoneIndexByTextPosition(index);
                if (sIndex != -1) {
                    for (ISegmentListener listener : myListeners) {
                        listener.segmentClicked(sIndex, true);
                        repaint();
                    }
                }
            }

            public void mousePressed(MouseEvent me) {
                int index = getIndexByPoint(me.getPoint());
                if (index == -1) {
                    return;
                }
                int sIndex = myTextInfo.getSegmentIndexByTextPosition(index);
                if (sIndex == -1) {
                    sIndex = myTextInfo.getDeadZoneIndexByTextPosition(index);
                    if (sIndex == -1) {
                        myHighlightBegin[0] = (me.getX() - MY_X_INDENT) / myCharWidth;
                        myHighlightBegin[1] = (me.getY() - MY_Y_INDENT) / myLineHeight;
                        myHighlightEnd[0] = myHighlightBegin[0];
                        myHighlightEnd[1] = myHighlightBegin[1];
                        makeBounds(index);
                        repaint();
                    }
                }
            }

            private void makeBounds(int index) {
                int[] bounds = getBounds(index);
                int i = 0;
                for (ILineInfo info : myLineInfo) {
                    if (info.getStartIndex() <= bounds[0] && (info.getStartIndex()
                            + info.getLength() >= bounds[0])) {
                        myHighlightMinPossible[1] = i;
                        myHighlightMinPossible[0] = bounds[0] - info.getStartIndex();
                    }
                    if (info.getStartIndex() <= bounds[1] && (info.getStartIndex()
                            + info.getLength() >= bounds[1])) {
                        myHighlightMaxPossible[1] = i;
                        myHighlightMaxPossible[0] = bounds[1] - info.getStartIndex();
                        return;
                    }
                    i++;
                }

                myHighlightMaxPossible[1] = myLineInfo.size();
                myHighlightMaxPossible[0] = 1;

            }

            private int[] getBounds(int index) {
                int start = 0;
                int end = Integer.MAX_VALUE;

                for (ISegment s: myTextInfo.getDeadZones()) {
                    if (s.getEnd() < index) {
                        start = s.getEnd();
                    }
                    if (s.getStart() > index) {
                        end = s. getStart() - 1;
                        break;
                    }
                }
                for (ISegment s: myTextInfo.getSegments()) {
                    if ((s.getEnd() < index) && (s.getEnd() > start)) {
                        start = s.getEnd();
                    }
                    if (s.getStart() > index) {
                        if (s.getStart() < end) {
                            end = s. getStart() - 1;
                        }
                        break;
                    }
                }
                return new int[] {start, end};
            }

            public void mouseDragged(MouseEvent e) {
                int i = (e.getX() - MY_X_INDENT) / myCharWidth;
                int j = (e.getY() - MY_Y_INDENT) / myLineHeight;

                if (j >= myLineInfo.size()) {
                    myHighlightEnd[0] = myHighlightMaxPossible[0];
                    myHighlightEnd[1] = myHighlightMaxPossible[1];
                    return;
                }

                ILineInfo lineIfo = myLineInfo.get(j);
                i = (i >= lineIfo.getLength()) ? lineIfo.getLength() - 1 : i;

                if (j < myHighlightMinPossible[1] || (j == myHighlightMinPossible[1] &&
                        i < myHighlightMinPossible[0])) {
                    myHighlightEnd[0] = myHighlightMinPossible[0];
                    myHighlightEnd[1] = myHighlightMinPossible[1];
                } else if (j > myHighlightMaxPossible[1] || (j == myHighlightMaxPossible[1] &&
                        i > myHighlightMaxPossible[0])) {
                    myHighlightEnd[0] = myHighlightMaxPossible[0];
                    myHighlightEnd[1] = myHighlightMaxPossible[1];
                } else {
                    myHighlightEnd[0] = i;
                    myHighlightEnd[1] = j;
                }
                repaint();
            }

            public void mouseMoved(MouseEvent e) {

            }

            public void mouseReleased(MouseEvent me) {
                if (myHighlightBegin[1] == -1) {
                    return;
                }
                int i = myLineInfo.get(myHighlightBegin[1]).getStartIndex() + myHighlightBegin[0];
                int j = myLineInfo.get(myHighlightEnd[1]).getStartIndex() + myHighlightEnd[0];

                for (ISegmentListener listener : myListeners) {
                    listener.segmentCreated(Math.min(i, j), Math.max(i,j) + 1);
                    repaint();
                }
                dropHighlights();
                repaint();
            }
        }
    }
}