package ru.amse.cat.evlarchick.ui;

import ru.amse.cat.evlarchick.textstructure.ISegmentWithTranslation;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.*;

public class SegmentTranslationDialog extends JDialog {
    private ISegmentWithTranslation mySegment;
    private Color myColor1 = Color.PINK;
    private Color myBColor = Color.DARK_GRAY;
    private Color myTextColor = Color.BLACK;
    private Color myAreaColor = Color.WHITE;
    private Color myButtonColor = Color.DARK_GRAY;

    public SegmentTranslationDialog(final ISegmentWithTranslation segment) {
        Font font = new Font("Monospaced", Font.PLAIN, 12);

        final JTextArea translation = new JTextArea(segment.getTranslation());
        translation.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
        translation.setFont(font);
        translation.setColumns(60);
        translation.setRows(5);
        translation.invalidate();
        translation.setForeground(myTextColor);
        translation.setBackground(myAreaColor);
        translation.setLineWrap(true);
        translation.requestFocusInWindow();

        JTextArea original = new JTextArea(segment.getText());
        original.setEditable(false);
        original.setBorder(BorderFactory.createEmptyBorder(3, 3, 3, 3));
        original.setFont(font);
        original.setColumns(60);
        original.setRows(5);
        original.invalidate();
        original.setForeground(myTextColor);
        original.setBackground(myAreaColor);

        JButton okButton = new JButton("OK");
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                segment.setTranslation(translation.getText());
                dispose();
            }
        });
        okButton.setForeground(myColor1);
        okButton.setBackground(myButtonColor);
        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                dispose();
            }
        });
        cancelButton.setForeground(myColor1);
        cancelButton.setBackground(myButtonColor);

        JPanel oPanel = new JPanel();
        oPanel.add(new JScrollPane(original), BorderLayout.CENTER);
        oPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(myColor1)
                , "Original Text", TitledBorder.DEFAULT_JUSTIFICATION,
                TitledBorder.DEFAULT_POSITION, font, myColor1));
        oPanel.setBackground(myBColor);

        JPanel tPanel = new JPanel();
        tPanel.add(new JScrollPane(translation), BorderLayout.CENTER);
        tPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createLineBorder(myColor1)
                , "Translation", TitledBorder.DEFAULT_JUSTIFICATION,
                TitledBorder.DEFAULT_POSITION, font, myColor1));
        tPanel.setBackground(myBColor);

        JPanel textPanel = new JPanel();
        textPanel.setLayout(new BorderLayout());
        textPanel.add(oPanel, BorderLayout.NORTH);
        textPanel.add(tPanel, BorderLayout.SOUTH);
        textPanel.setBackground(myBColor);
        textPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(okButton, BorderLayout.WEST);
        buttonPanel.add(cancelButton, BorderLayout.EAST);
        buttonPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
        buttonPanel.setBackground(myBColor);

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(textPanel, BorderLayout.NORTH);
        mainPanel.add(buttonPanel, BorderLayout.SOUTH);
        mainPanel.setBackground(myBColor);

        setTitle("Traslation Making");
        setModal(true);
        setResizable(false);
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        setContentPane(mainPanel);
        pack();
    }
}
